/*	MikMod sound library
	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
	complete list.

	This library is free software; you can redistribute it and/or modify
	it under the terms of the GNU Library General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Library General Public License for more details.
 
	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

/*==============================================================================

  $Id: drv_ds.c,v 1.1 1999/03/24 11:43:15 miod Exp $

  Driver for output on win32 platforms using DirectSound

==============================================================================*/

/*

	Written by Brian McKinney <Brian.McKinney@colorado.edu>

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>

#define INITGUID
#include <dsound.h>

#include <mikmod_internals.h>

/* size of each buffer */
#define FRAGSIZE 262144
/* buffer count */
#define UPDATES  2
/* total buffer size */
#define BUFFERSIZE (FRAGSIZE*UPDATES)

static LPDIRECTSOUND pSoundCard;
static LPDIRECTSOUNDBUFFER pPrimarySoundBuffer,pSoundBuffer;
static LPDIRECTSOUNDNOTIFY pSoundBufferNotify;

static HANDLE notifyUpdateHandle,updateBufferHandle;
static DWORD updateBufferThreadID,soundBufferCurrentPosition,blockBytes1,blockBytes2;
static LPVOID pBlock1,pBlock2;
static DWORD sleepInterval;
static BOOL threadInUse;

static DWORD WINAPI updateBufferProc(LPVOID lpParameter)
{
	while(threadInUse) {
		if(WaitForSingleObject(notifyUpdateHandle,INFINITE)==WAIT_OBJECT_0) {
			pSoundBuffer->lpVtbl->GetCurrentPosition
			                    (pSoundBuffer,&soundBufferCurrentPosition,NULL);
		
			if(soundBufferCurrentPosition<FRAGSIZE) {
				if(pSoundBuffer->lpVtbl->Lock
				          (pSoundBuffer,FRAGSIZE,FRAGSIZE,&pBlock1,&blockBytes1,
				           &pBlock2,&blockBytes2,0)==DSERR_BUFFERLOST) {	
					pSoundBuffer->lpVtbl->Restore(pSoundBuffer);
					pSoundBuffer->lpVtbl->Lock
					      (pSoundBuffer,FRAGSIZE,FRAGSIZE,&pBlock1,&blockBytes1,
					       &pBlock2,&blockBytes2,0);
				}

				MUTEX_LOCK
					VC_WriteBytes((SBYTE*)pBlock1,(ULONG)blockBytes1);
					if(pBlock2)
						VC_WriteBytes((SBYTE*)pBlock2,(ULONG)blockBytes2);
				MUTEX_UNLOCK

				pSoundBuffer->lpVtbl->Unlock
				         (pSoundBuffer,pBlock1,blockBytes1,pBlock2,blockBytes2);
			} else {
				if(pSoundBuffer->lpVtbl->Lock
				                 (pSoundBuffer,0,FRAGSIZE,&pBlock1,&blockBytes1,
				                  &pBlock2,&blockBytes2,0)==DSERR_BUFFERLOST) {	
					pSoundBuffer->lpVtbl->Restore(pSoundBuffer);
					pSoundBuffer->lpVtbl->Lock
					             (pSoundBuffer,0,FRAGSIZE,&pBlock1,&blockBytes1,
					              &pBlock2,&blockBytes2,0);
				}

				if(Player_Paused()) {
					MUTEX_LOCK
						VC_SilenceBytes((SBYTE*)pBlock1,(ULONG)blockBytes1);
						if(pBlock2)
							VC_SilenceBytes((SBYTE*)pBlock2,(ULONG)blockBytes2);
					MUTEX_UNLOCK
				} else {
					MUTEX_LOCK
						VC_WriteBytes((SBYTE*)pBlock1,(ULONG)blockBytes1);
						if(pBlock2)
							VC_WriteBytes((SBYTE*)pBlock2,(ULONG)blockBytes2);
					MUTEX_UNLOCK
				}
			
				pSoundBuffer->lpVtbl->Unlock
				         (pSoundBuffer,pBlock1,blockBytes1,pBlock2,blockBytes2);
			}
		}
		Sleep(sleepInterval);
	}
	return 0;
}

static BOOL DS_IsPresent(void)
{
	if(DirectSoundCreate(NULL,&pSoundCard,NULL)!=DS_OK)
		return 0;
	if(pSoundCard)
		pSoundCard->lpVtbl->Release(pSoundCard);
	return 1;
}

static BOOL DS_Init(void)
{
	DSBUFFERDESC soundBufferFormat;
	WAVEFORMATEX pcmwf;
	DSBPOSITIONNOTIFY positionNotifications[2];

	if(DirectSoundCreate(NULL,&pSoundCard,NULL)!=DS_OK) {
		_mm_errno=MMERR_OPENING_AUDIO;
		return 1;
	}

	if(pSoundCard->lpVtbl->SetCooperativeLevel
	                 (pSoundCard,GetForegroundWindow(),DSSCL_PRIORITY)!=DS_OK) {
		_mm_errno=MMERR_DS_PRIORITY;
		return 1;
	}
	
	memset(&pcmwf,0,sizeof(PCMWAVEFORMAT));
    pcmwf.wFormatTag     =WAVE_FORMAT_PCM;
    pcmwf.nChannels      =(md_mode&DMODE_STEREO)?2:1;
    pcmwf.nSamplesPerSec =md_mixfreq;
    pcmwf.nBlockAlign    =4;
    pcmwf.nAvgBytesPerSec=pcmwf.nSamplesPerSec*pcmwf.nBlockAlign;
    pcmwf.wBitsPerSample =(md_mode&DMODE_16BITS)?16:8;
   
	memset(&soundBufferFormat,0,sizeof(DSBUFFERDESC));
    soundBufferFormat.dwSize       =sizeof(DSBUFFERDESC);
    soundBufferFormat.dwFlags      =DSBCAPS_PRIMARYBUFFER;
    soundBufferFormat.dwBufferBytes=0;                                
    soundBufferFormat.lpwfxFormat  =NULL; 
	
	if(pSoundCard->lpVtbl->CreateSoundBuffer
	         (pSoundCard,&soundBufferFormat,&pPrimarySoundBuffer,NULL)!=DS_OK) {
		_mm_errno=MMERR_DS_BUFFER;
		return 1;
	}

    if(pPrimarySoundBuffer->lpVtbl->SetFormat
	                                      (pPrimarySoundBuffer,&pcmwf)!=DS_OK) {
		_mm_errno=MMERR_DS_FORMAT;
		return 1;
	}
    pPrimarySoundBuffer->lpVtbl->Play(pPrimarySoundBuffer,0,0,DSBPLAY_LOOPING);
   	
	memset(&pcmwf,0,sizeof(PCMWAVEFORMAT));
    pcmwf.wFormatTag     =WAVE_FORMAT_PCM;
    pcmwf.nChannels      =(md_mode&DMODE_STEREO)?2:1;
    pcmwf.nSamplesPerSec =md_mixfreq;
    pcmwf.nBlockAlign    =4;
    pcmwf.nAvgBytesPerSec=pcmwf.nSamplesPerSec*pcmwf.nBlockAlign;
    pcmwf.wBitsPerSample =(md_mode&DMODE_16BITS)?16:8;

	sleepInterval=((FRAGSIZE)/pcmwf.nAvgBytesPerSec)*1000;

	memset(&soundBufferFormat,0,sizeof(DSBUFFERDESC));
    soundBufferFormat.dwSize       =sizeof(DSBUFFERDESC);
    soundBufferFormat.dwFlags      =DSBCAPS_CTRLALL|DSBCAPS_GETCURRENTPOSITION2 ;
    soundBufferFormat.dwBufferBytes=BUFFERSIZE;                                
    soundBufferFormat.lpwfxFormat  =&pcmwf;
	
	if(pSoundCard->lpVtbl->CreateSoundBuffer
	                (pSoundCard,&soundBufferFormat,&pSoundBuffer,NULL)!=DS_OK) {
		_mm_errno=MMERR_DS_BUFFER;
		return 1;
	}

	pSoundBuffer->lpVtbl->QueryInterface
	        (pSoundBuffer,&IID_IDirectSoundNotify,(LPVOID*)&pSoundBufferNotify);
	if(!pSoundBufferNotify) {
		_mm_errno=MMERR_DS_NOTIFY;
		return 1;
	}

	notifyUpdateHandle=CreateEvent
	     (NULL,FALSE,FALSE,"libmikmod DirectSound Driver positionNotify Event");
	if(!notifyUpdateHandle) {
		_mm_errno=MMERR_DS_EVENT;
		return 1;
	}

	updateBufferHandle=CreateThread
	      (NULL,0,updateBufferProc,NULL,CREATE_SUSPENDED,&updateBufferThreadID);
	if(!updateBufferHandle) {
		_mm_errno=MMERR_DS_THREAD;
		return 1;
	}

	memset(&positionNotifications,0,2*sizeof(DSBPOSITIONNOTIFY));
	positionNotifications[0].dwOffset    =0;
	positionNotifications[0].hEventNotify=notifyUpdateHandle;
	positionNotifications[1].dwOffset    =FRAGSIZE;
	positionNotifications[1].hEventNotify=notifyUpdateHandle;
	if(pSoundBufferNotify->lpVtbl->SetNotificationPositions
	                     (pSoundBufferNotify,2,&positionNotifications)!=DS_OK) {
		_mm_errno=MMERR_DS_UPDATE;
		return 1;
	}
	
	if(VC_Init())
		return 1;

	return 0;
}

static void DS_Exit(void)
{
	DWORD statusInfo;

	if(updateBufferHandle) {
		CloseHandle((HANDLE)updateBufferThreadID);
		CloseHandle(updateBufferHandle);
	}
	CloseHandle(notifyUpdateHandle);
	
	VC_Exit();

	if(pSoundBufferNotify)
		pSoundBufferNotify->lpVtbl->Release(pSoundBufferNotify);
	if(pSoundBuffer) {
		if(pSoundBuffer->lpVtbl->GetStatus(pSoundBuffer,&statusInfo)==DS_OK)
			if(statusInfo&DSBSTATUS_PLAYING)
				pSoundBuffer->lpVtbl->Stop(pSoundBuffer);
		pSoundBuffer->lpVtbl->Release(pSoundBuffer);
	}

	if(pPrimarySoundBuffer) {
		if(pPrimarySoundBuffer->lpVtbl->GetStatus
		                               (pPrimarySoundBuffer,&statusInfo)==DS_OK)
			if(statusInfo&DSBSTATUS_PLAYING)
				pPrimarySoundBuffer->lpVtbl->Stop(pSoundBuffer);
		pPrimarySoundBuffer->lpVtbl->Release(pPrimarySoundBuffer);
	}

	if(pSoundCard)
		pSoundCard->lpVtbl->Release(pSoundCard);
}

static void DS_Update(void)
{
	return;
}

static void DS_PlayStop(void)
{
	pSoundBuffer->lpVtbl->Stop(pSoundBuffer);
	threadInUse=0;
	VC_PlayStop();
}

static BOOL DS_PlayStart(void)
{
	pSoundBuffer->lpVtbl->Play(pSoundBuffer,0,0,DSBPLAY_LOOPING);
	threadInUse=1;
	ResumeThread(updateBufferHandle);
	VC_PlayStart();

	return 0;
}

MDRIVER drv_ds=
{
	NULL,
	"DirectSound",
	"DirectSound Driver (DX6) v0.1",
	0,
	255,
	DS_IsPresent,
	VC_SampleLoad,
	VC_SampleUnload,
	VC_SampleSpace,
	VC_SampleLength,
	DS_Init,
	DS_Exit,
	NULL,
	VC_SetNumVoices,
	DS_PlayStart,
	DS_PlayStop,
	DS_Update,
	NULL,
	VC_VoiceSetVolume,
	VC_VoiceGetVolume,
	VC_VoiceSetFrequency,
	VC_VoiceGetFrequency,
	VC_VoiceSetPanning,
	VC_VoiceGetPanning,
	VC_VoicePlay,
	VC_VoiceStop,
	VC_VoiceStopped,
	VC_VoiceGetPosition,
	VC_VoiceRealVolume
};

/* ex:set ts=4: */
