/*	MikMod sound library
	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
	complete list.

	This library is free software; you can redistribute it and/or modify
	it under the terms of the GNU Library General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Library General Public License for more details.
 
	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

/*==============================================================================

  $Id: drv_sun.c,v 1.24 1999/03/24 06:00:05 miod Exp $

  Driver for output on the Sun audio device (/dev/audio).
  Also works under NetBSD and OpenBSD

==============================================================================*/

/*

	Written by Valtteri Vuorikoski <vuori@sci.fi>

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <stdlib.h>
#if !defined(SUNOS)
#include <string.h>
#endif
#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#include <sys/types.h>

#ifdef SUNOS
#include <sun/audioio.h>
#else /* Solaris, *BSD */
#include <sys/audioio.h>
#endif

#include <mikmod_internals.h>

#define DEFAULT_FRAGSIZE 4096

static	int sndfd=-1;
static	int play_encoding;
static	int play_precision;
static	int fragsize=DEFAULT_FRAGSIZE;
static	SBYTE *audiobuffer=NULL;

/* Sound device to open */
#if defined(__NetBSD__)||defined(__OpenBSD__)
#define SOUNDDEVICE "/dev/audio"
#else
#define SOUNDDEVICE "/dev/sound"
#endif

/* Compatibility defines, for old *BSD or SunOS systems */
#ifndef AUDIO_ENCODING_PCM16
#define AUDIO_ENCODING_PCM16 AUDIO_ENCODING_LINEAR
#endif
#ifndef AUDIO_ENCODING_PCM8
#define AUDIO_ENCODING_PCM8 AUDIO_ENCODING_LINEAR8
#endif
#ifndef AUDIO_ENCODING_SLINEAR_LE
#define AUDIO_ENCODING_SLINEAR_LE AUDIO_ENCODING_PCM16
#endif
#ifndef AUDIO_ENCODING_ULINEAR_LE
#define AUDIO_ENCODING_ULINEAR_LE AUDIO_ENCODING_PCM8
#endif

/*
This is modified from the SOX package (libst.h and raw.c) by Lance Norskog.
Parts of this are...
** Copyright (C) 1989 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/
#define st_linear_to_ulaw(linearword) ulaw_comp_table[(linearword/4)&0x3fff]

static unsigned char ulaw_comp_table[];

/* this converts unsigned linear data from VC_WriteBytes() to ulaw or signed
   linear */
static void unsignedtoulaw(char *buf,int nsamp)
{
	while(nsamp--) {
		register long datum=(long)*((unsigned char*)buf);

		datum^=128; /* convert to signed */
		datum<<=8;
		/* round up to 12 bits of data */
		datum+=0x8;	/* + 0b1000 */
		datum=st_linear_to_ulaw(datum);
		*buf++=(char)datum;
	}
}
/* end of soxism */

static BOOL Sun_IsThere(void)
{
	return (access(SOUNDDEVICE,W_OK)==0);
}

static BOOL Sun_Init(void)
{
	int play_stereo,play_rate;
#ifdef SUNOS
	int audiotype;
#else
	audio_device_t audiotype;	
#endif
	struct audio_info audioinfo;
	char *env;

	if((sndfd=open(SOUNDDEVICE,O_WRONLY))<0) {
		_mm_errno=MMERR_OPENING_AUDIO;
		return 1;
	}

	if((env=getenv("MM_FRAGSIZE"))) fragsize=atoi(env);
	if(fragsize<128) fragsize=DEFAULT_FRAGSIZE;

	if(!(audiobuffer=(SBYTE*)_mm_malloc(fragsize)))
		return 1;

	play_precision=(md_mode&DMODE_16BITS)?16:8;
	play_stereo=(md_mode&DMODE_STEREO)?2:1;
	play_rate=md_mixfreq;
	/* attempt to guess the encoding */
	play_encoding=-1;

	if(ioctl(sndfd,AUDIO_GETDEV,&audiotype)<0) {
#ifdef MIKMOD_DEBUG
		fputs("\rSun driver warning: could not determine audio device type\n",stderr);
#endif
	} else {
#if !defined(__NetBSD__)&&!defined(__OpenBSD__) /* SunOS 4, Solaris */
#ifdef SUNOS /* SunOS 4 */
		switch(audiotype) {
			case AUDIO_DEV_AMD:
				/* AMD 79C30 */
				/* 8bit mono ulaw 8kHz */
				if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
					play_encoding=AUDIO_ENCODING_ULAW;
				else {
					_mm_errno=MMERR_SUN_INIT;
					return 1;
				}
				break;
			case AUDIO_DEV_SPEAKERBOX:
			case AUDIO_DEV_CODEC:
				/* CS 4231 or DBRI or speaker box */
				/* 16bit mono/stereo linear 8kHz - 48kHz */
				if(play_precision==16)
					play_encoding=AUDIO_ENCODING_LINEAR;
				/* 8bit mono ulaw 8kHz - 48kHz */
				else if((play_precision==8)&&(play_stereo==1))
					play_encoding=AUDIO_ENCODING_ULAW;
				else {
					_mm_errno=MMERR_SUN_INIT;
					return 1;
				}
				break;
		}
#else /* Solaris */
		if (!strcmp(audiotype.name,"SUNW,am79c30")) {
			/* AMD 79C30 */
			/* 8bit mono ulaw 8kHz */
			if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		} else
		  if ((!strcmp(audiotype.name,"SUNW,CS4231"))||
		      (!strcmp(audiotype.name,"SUNW,dbri"))||
		      (!strcmp(audiotype.name,"speakerbox"))) {
			/* CS 4231 or DBRI or speaker box */
			/* 16bit mono/stereo linear 8kHz - 48kHz */
			if(play_precision==16)
				play_encoding=AUDIO_ENCODING_LINEAR;
			/* 8bit mono ulaw 8kHz - 48kHz */
			else if((play_precision==8)&&(play_stereo==1))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		}
#endif
#else /* NetBSD, OpenBSD */
		if (!strcmp(audiotype.name,"amd7930")) {
			/* AMD 79C30 */
			/* 8bit mono ulaw 8kHz */
			if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		}
		if(!strcmp(audiotype.name,"Am78C201")) {
			/* AMD Interwave */
			/* 16bit stereo linear 44kHz */
			if((play_precision!=8)||(play_stereo!=2)||(play_rate!=44100)) {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			} else
				play_encoding=AUDIO_ENCODING_SLINEAR_LE;
		}
#endif
	}

	/* Sound devices which were not handled above don't have specific
	   limitations, so try and guess optimal settings */
	if(play_encoding==-1) {
		if((play_precision==8)&&(play_stereo==1)&&(play_rate<=8000))
			play_encoding = AUDIO_ENCODING_ULAW;
		else
			play_encoding=(play_precision==16)?AUDIO_ENCODING_SLINEAR_LE:
			                                   AUDIO_ENCODING_ULINEAR_LE;
	}

	AUDIO_INITINFO(&audioinfo);
	audioinfo.play.precision=play_precision;
	audioinfo.play.channels=play_stereo;
	audioinfo.play.sample_rate=play_rate;
	audioinfo.play.encoding=play_encoding;
#if !defined(__NetBSD__)&&!defined(__OpenBSD__) /* SunOS 4, Solaris */
#ifdef USE_HEADPHONE
	audioinfo.play.port=AUDIO_HEADPHONE;
#else
	audioinfo.play.port=AUDIO_SPEAKER;
#endif
#endif
#if defined(__NetBSD__)||defined(__OpenBSD__)
	audioinfo.mode=AUMODE_PLAY;
#endif
	
	if(ioctl(sndfd,AUDIO_SETINFO,&audioinfo)<0) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}

	/* Check if our changes were accepted */
	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}
	if ((audioinfo.play.precision!=play_precision)||
	    (audioinfo.play.channels!=play_stereo)||
	    (audioinfo.play.encoding!=play_encoding)||
	    (audioinfo.play.sample_rate!=play_rate)) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}

	return VC_Init();
}

static void Sun_Exit(void)
{
	VC_Exit();
	if (audiobuffer) {
		free(audiobuffer);
		audiobuffer=NULL;
	}
	if (sndfd>=0) {
		close(sndfd);
		sndfd=-1;
	}
}

static void Sun_Update(void)
{
	int done;

	done=VC_WriteBytes((signed char*)audiobuffer,fragsize);
	if(play_encoding==AUDIO_ENCODING_ULAW) unsignedtoulaw(audiobuffer,done);
	write(sndfd,audiobuffer,done);
}

static void Sun_Pause(void)
{
	int done;

	done=VC_SilenceBytes((signed char*)audiobuffer,fragsize);
	write(sndfd,audiobuffer,done);
}

static BOOL Sun_Reset(void)
{
	Sun_Exit();
	return Sun_Init();
}

static BOOL Sun_PlayStart(void)
{
	struct audio_info audioinfo;

	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0)
		return 1;
	audioinfo.play.pause=0;
	if(ioctl(sndfd,AUDIO_SETINFO,&audioinfo)<0)
		return 1;

	return VC_PlayStart();
}

static void Sun_PlayStop(void)
{
	struct audio_info audioinfo;

	VC_PlayStop();

	if(ioctl(sndfd,AUDIO_DRAIN)<0)
		return;
	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0)
		return;
	audioinfo.play.pause=1;
	ioctl(sndfd,AUDIO_SETINFO,&audioinfo);
}

MDRIVER drv_sun={
	NULL,
#ifdef __OpenBSD__
	"OpenBSD Audio",
	"OpenBSD Audio driver v0.4",
#else
#ifdef __NetBSD__
	"NetBSD Audio",
	"NetBSD Audio driver v0.4",
#else
	"Sun Audio",
	"Sun Audio driver v1.2",
#endif
#endif
	0,255,
	Sun_IsThere,
	VC_SampleLoad,
	VC_SampleUnload,
	VC_SampleSpace,
	VC_SampleLength,
	Sun_Init,
	Sun_Exit,
	Sun_Reset,
	VC_SetNumVoices,
	Sun_PlayStart,
	Sun_PlayStop,
	Sun_Update,
	Sun_Pause,
	VC_VoiceSetVolume,
	VC_VoiceGetVolume,
	VC_VoiceSetFrequency,
	VC_VoiceGetFrequency,
	VC_VoiceSetPanning,
	VC_VoiceGetPanning,
	VC_VoicePlay,
	VC_VoiceStop,
	VC_VoiceStopped,
	VC_VoiceGetPosition,
	VC_VoiceRealVolume
};

/* this table is used to convert signed linear data to ulaw 
   this is part of libst.c from the SOX package by Lance Norskog. */
static unsigned char ulaw_comp_table[16384] = {
	0xff,0xfe,0xfe,0xfd,0xfd,0xfc,0xfc,0xfb,
	0xfb,0xfa,0xfa,0xf9,0xf9,0xf8,0xf8,0xf7,
	0xf7,0xf6,0xf6,0xf5,0xf5,0xf4,0xf4,0xf3,
	0xf3,0xf2,0xf2,0xf1,0xf1,0xf0,0xf0,0xef,
	0xef,0xef,0xef,0xee,0xee,0xee,0xee,0xed,
	0xed,0xed,0xed,0xec,0xec,0xec,0xec,0xeb,
	0xeb,0xeb,0xeb,0xea,0xea,0xea,0xea,0xe9,
	0xe9,0xe9,0xe9,0xe8,0xe8,0xe8,0xe8,0xe7,
	0xe7,0xe7,0xe7,0xe6,0xe6,0xe6,0xe6,0xe5,
	0xe5,0xe5,0xe5,0xe4,0xe4,0xe4,0xe4,0xe3,
	0xe3,0xe3,0xe3,0xe2,0xe2,0xe2,0xe2,0xe1,
	0xe1,0xe1,0xe1,0xe0,0xe0,0xe0,0xe0,0xdf,
	0xdf,0xdf,0xdf,0xdf,0xdf,0xdf,0xdf,0xde,
	0xde,0xde,0xde,0xde,0xde,0xde,0xde,0xdd,
	0xdd,0xdd,0xdd,0xdd,0xdd,0xdd,0xdd,0xdc,
	0xdc,0xdc,0xdc,0xdc,0xdc,0xdc,0xdc,0xdb,
	0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xda,
	0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xd9,
	0xd9,0xd9,0xd9,0xd9,0xd9,0xd9,0xd9,0xd8,
	0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd7,
	0xd7,0xd7,0xd7,0xd7,0xd7,0xd7,0xd7,0xd6,
	0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd5,
	0xd5,0xd5,0xd5,0xd5,0xd5,0xd5,0xd5,0xd4,
	0xd4,0xd4,0xd4,0xd4,0xd4,0xd4,0xd4,0xd3,
	0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0xd2,
	0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xd1,
	0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd0,
	0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xcf,
	0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,
	0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xce,
	0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xce,
	0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xcd,
	0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,
	0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcc,
	0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,
	0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcc,0xcb,
	0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,
	0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xca,
	0xca,0xca,0xca,0xca,0xca,0xca,0xca,0xca,
	0xca,0xca,0xca,0xca,0xca,0xca,0xca,0xc9,
	0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,
	0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc8,
	0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,
	0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc7,
	0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,
	0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc6,
	0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,
	0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0xc5,
	0xc5,0xc5,0xc5,0xc5,0xc5,0xc5,0xc5,0xc5,
	0xc5,0xc5,0xc5,0xc5,0xc5,0xc5,0xc5,0xc4,
	0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,
	0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc3,
	0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,
	0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,0xc2,
	0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
	0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc1,
	0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,
	0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc0,
	0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,
	0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xbf,
	0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
	0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
	0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
	0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbe,
	0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,
	0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,
	0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,
	0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0xbd,
	0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,
	0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,
	0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,
	0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbd,0xbc,
	0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,
	0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,
	0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,
	0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbc,0xbb,
	0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,
	0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,
	0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,
	0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xba,
	0xba,0xba,0xba,0xba,0xba,0xba,0xba,0xba,
	0xba,0xba,0xba,0xba,0xba,0xba,0xba,0xba,
	0xba,0xba,0xba,0xba,0xba,0xba,0xba,0xba,
	0xba,0xba,0xba,0xba,0xba,0xba,0xba,0xb9,
	0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,
	0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,
	0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,
	0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb9,0xb8,
	0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,
	0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,
	0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,
	0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb8,0xb7,
	0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,
	0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,
	0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,
	0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb6,
	0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,
	0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,
	0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,
	0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb6,0xb5,
	0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,
	0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,
	0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,
	0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb5,0xb4,
	0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,
	0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,
	0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,
	0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb4,0xb3,
	0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,
	0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,
	0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,
	0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb2,
	0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
	0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
	0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
	0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb1,
	0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,
	0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,
	0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,
	0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb1,0xb0,
	0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
	0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
	0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
	0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,
	0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xae,
	0xae,0xae,0xae,0xae,0xae,0xae,0xae,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
	0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xac,
	0xac,0xac,0xac,0xac,0xac,0xac,0xac,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
	0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
	0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,
	0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa9,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,
	0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa8,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,
	0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,
	0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa6,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,
	0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa5,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
	0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,
	0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa3,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
	0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,
	0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa1,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,
	0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0xa0,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,
	0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9f,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,
	0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9e,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
	0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,
	0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,
	0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,
	0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x9a,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x99,
	0x99,0x99,0x99,0x99,0x99,0x99,0x99,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
	0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
	0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x96,
	0x96,0x96,0x96,0x96,0x96,0x96,0x96,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x95,
	0x95,0x95,0x95,0x95,0x95,0x95,0x95,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,
	0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
	0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,
	0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,
	0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,
	0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8f,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
	0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,
	0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8d,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,
	0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,
	0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8b,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,
	0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x8a,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x89,
	0x89,0x89,0x89,0x89,0x89,0x89,0x89,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x87,
	0x87,0x87,0x87,0x87,0x87,0x87,0x87,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x86,
	0x86,0x86,0x86,0x86,0x86,0x86,0x86,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x85,
	0x85,0x85,0x85,0x85,0x85,0x85,0x85,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x84,
	0x84,0x84,0x84,0x84,0x84,0x84,0x84,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x83,
	0x83,0x83,0x83,0x83,0x83,0x83,0x83,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x81,
	0x81,0x81,0x81,0x81,0x81,0x81,0x81,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
	0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
	0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
	0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x04,0x04,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
	0x05,0x05,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
	0x06,0x06,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
	0x08,0x08,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
	0x09,0x09,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,
	0x0a,0x0a,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,0x0b,
	0x0b,0x0b,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
	0x0c,0x0c,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,
	0x0d,0x0d,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,0x0e,
	0x0e,0x0e,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
	0x0f,0x0f,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10,
	0x10,0x10,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
	0x11,0x11,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12,
	0x12,0x12,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
	0x13,0x13,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x14,0x14,0x14,0x14,0x14,0x14,
	0x14,0x14,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,
	0x15,0x15,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16,
	0x16,0x16,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
	0x17,0x17,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
	0x18,0x18,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x19,0x19,0x19,0x19,0x19,0x19,
	0x19,0x19,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,0x1a,
	0x1a,0x1a,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,0x1b,
	0x1b,0x1b,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,
	0x1c,0x1c,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,
	0x1d,0x1d,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
	0x1e,0x1e,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,
	0x1f,0x1f,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
	0x20,0x20,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
	0x21,0x21,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
	0x22,0x22,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x23,0x23,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x24,
	0x24,0x24,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x25,
	0x25,0x25,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,
	0x26,0x26,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,
	0x27,0x27,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28,
	0x28,0x28,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x29,0x29,0x29,0x29,0x29,0x29,
	0x29,0x29,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
	0x2a,0x2a,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,0x2b,
	0x2b,0x2b,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
	0x2c,0x2c,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,0x2d,
	0x2d,0x2d,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,0x2e,
	0x2e,0x2e,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,0x2f,
	0x2f,0x2f,0x30,0x30,0x30,0x30,0x30,0x30,
	0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
	0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
	0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
	0x30,0x30,0x31,0x31,0x31,0x31,0x31,0x31,
	0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,
	0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,
	0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,
	0x31,0x31,0x32,0x32,0x32,0x32,0x32,0x32,
	0x32,0x32,0x32,0x32,0x32,0x32,0x32,0x32,
	0x32,0x32,0x32,0x32,0x32,0x32,0x32,0x32,
	0x32,0x32,0x32,0x32,0x32,0x32,0x32,0x32,
	0x32,0x32,0x33,0x33,0x33,0x33,0x33,0x33,
	0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
	0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
	0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
	0x33,0x33,0x34,0x34,0x34,0x34,0x34,0x34,
	0x34,0x34,0x34,0x34,0x34,0x34,0x34,0x34,
	0x34,0x34,0x34,0x34,0x34,0x34,0x34,0x34,
	0x34,0x34,0x34,0x34,0x34,0x34,0x34,0x34,
	0x34,0x34,0x35,0x35,0x35,0x35,0x35,0x35,
	0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
	0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
	0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
	0x35,0x35,0x36,0x36,0x36,0x36,0x36,0x36,
	0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
	0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
	0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
	0x36,0x36,0x37,0x37,0x37,0x37,0x37,0x37,
	0x37,0x37,0x37,0x37,0x37,0x37,0x37,0x37,
	0x37,0x37,0x37,0x37,0x37,0x37,0x37,0x37,
	0x37,0x37,0x37,0x37,0x37,0x37,0x37,0x37,
	0x37,0x37,0x38,0x38,0x38,0x38,0x38,0x38,
	0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
	0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
	0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
	0x38,0x38,0x39,0x39,0x39,0x39,0x39,0x39,
	0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,
	0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,
	0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,
	0x39,0x39,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,
	0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,
	0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,
	0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,
	0x3a,0x3a,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,
	0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,
	0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,
	0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,
	0x3b,0x3b,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,
	0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,
	0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,
	0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,
	0x3c,0x3c,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,
	0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,
	0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,
	0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,
	0x3d,0x3d,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,
	0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,
	0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,
	0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,
	0x3e,0x3e,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
	0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
	0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
	0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
	0x3f,0x3f,0x40,0x40,0x40,0x40,0x40,0x40,
	0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x40,
	0x40,0x40,0x41,0x41,0x41,0x41,0x41,0x41,
	0x41,0x41,0x41,0x41,0x41,0x41,0x41,0x41,
	0x41,0x41,0x42,0x42,0x42,0x42,0x42,0x42,
	0x42,0x42,0x42,0x42,0x42,0x42,0x42,0x42,
	0x42,0x42,0x43,0x43,0x43,0x43,0x43,0x43,
	0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,
	0x43,0x43,0x44,0x44,0x44,0x44,0x44,0x44,
	0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,
	0x44,0x44,0x45,0x45,0x45,0x45,0x45,0x45,
	0x45,0x45,0x45,0x45,0x45,0x45,0x45,0x45,
	0x45,0x45,0x46,0x46,0x46,0x46,0x46,0x46,
	0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,
	0x46,0x46,0x47,0x47,0x47,0x47,0x47,0x47,
	0x47,0x47,0x47,0x47,0x47,0x47,0x47,0x47,
	0x47,0x47,0x48,0x48,0x48,0x48,0x48,0x48,
	0x48,0x48,0x48,0x48,0x48,0x48,0x48,0x48,
	0x48,0x48,0x49,0x49,0x49,0x49,0x49,0x49,
	0x49,0x49,0x49,0x49,0x49,0x49,0x49,0x49,
	0x49,0x49,0x4a,0x4a,0x4a,0x4a,0x4a,0x4a,
	0x4a,0x4a,0x4a,0x4a,0x4a,0x4a,0x4a,0x4a,
	0x4a,0x4a,0x4b,0x4b,0x4b,0x4b,0x4b,0x4b,
	0x4b,0x4b,0x4b,0x4b,0x4b,0x4b,0x4b,0x4b,
	0x4b,0x4b,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
	0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,
	0x4c,0x4c,0x4d,0x4d,0x4d,0x4d,0x4d,0x4d,
	0x4d,0x4d,0x4d,0x4d,0x4d,0x4d,0x4d,0x4d,
	0x4d,0x4d,0x4e,0x4e,0x4e,0x4e,0x4e,0x4e,
	0x4e,0x4e,0x4e,0x4e,0x4e,0x4e,0x4e,0x4e,
	0x4e,0x4e,0x4f,0x4f,0x4f,0x4f,0x4f,0x4f,
	0x4f,0x4f,0x4f,0x4f,0x4f,0x4f,0x4f,0x4f,
	0x4f,0x4f,0x50,0x50,0x50,0x50,0x50,0x50,
	0x50,0x50,0x51,0x51,0x51,0x51,0x51,0x51,
	0x51,0x51,0x52,0x52,0x52,0x52,0x52,0x52,
	0x52,0x52,0x53,0x53,0x53,0x53,0x53,0x53,
	0x53,0x53,0x54,0x54,0x54,0x54,0x54,0x54,
	0x54,0x54,0x55,0x55,0x55,0x55,0x55,0x55,
	0x55,0x55,0x56,0x56,0x56,0x56,0x56,0x56,
	0x56,0x56,0x57,0x57,0x57,0x57,0x57,0x57,
	0x57,0x57,0x58,0x58,0x58,0x58,0x58,0x58,
	0x58,0x58,0x59,0x59,0x59,0x59,0x59,0x59,
	0x59,0x59,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,
	0x5a,0x5a,0x5b,0x5b,0x5b,0x5b,0x5b,0x5b,
	0x5b,0x5b,0x5c,0x5c,0x5c,0x5c,0x5c,0x5c,
	0x5c,0x5c,0x5d,0x5d,0x5d,0x5d,0x5d,0x5d,
	0x5d,0x5d,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
	0x5e,0x5e,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,
	0x5f,0x5f,0x60,0x60,0x60,0x60,0x61,0x61,
	0x61,0x61,0x62,0x62,0x62,0x62,0x63,0x63,
	0x63,0x63,0x64,0x64,0x64,0x64,0x65,0x65,
	0x65,0x65,0x66,0x66,0x66,0x66,0x67,0x67,
	0x67,0x67,0x68,0x68,0x68,0x68,0x69,0x69,
	0x69,0x69,0x6a,0x6a,0x6a,0x6a,0x6b,0x6b,
	0x6b,0x6b,0x6c,0x6c,0x6c,0x6c,0x6d,0x6d,
	0x6d,0x6d,0x6e,0x6e,0x6e,0x6e,0x6f,0x6f,
	0x6f,0x6f,0x70,0x70,0x71,0x71,0x72,0x72,
	0x73,0x73,0x74,0x74,0x75,0x75,0x76,0x76,
	0x77,0x77,0x78,0x78,0x79,0x79,0x7a,0x7a,
	0x7b,0x7b,0x7c,0x7c,0x7d,0x7d,0x7e,0x7e};

/* ex:set ts=4: */
